#!/usr/bin/perl
#
# create_man_usr_file.pl
#
# This script creates IDE64 MAN configuration file ("man,usr") based on the provided data.
#
# Different sections for the following keys are separated by "=": "RETURN", "2", "3", "4".
#
# Special characters:
#   \{81} -> PETSCII single byte value of $81
#   _     -> space character byte value of $20
#
# Usage: perl create_man_usr_file.pl <CONFIGURATION.TXT>

use strict;
use warnings;

use bytes;
use open IN => ':bytes', OUT => ':bytes';

my $file = shift;

die 'File "man,usr" already exists' if -e "man,usr";
die "Usage: $0 <CONFIGURATION.TXT>" unless defined $file;
die "Configuration file \"${file}\" does not exists" unless -e $file;

my $byte;
our @data;
open CONF, $file;
binmode CONF;
push @data, sprintf ("%02x", 0x00), sprintf ("%02x", 0x10);
while ( sysread CONF, $byte, 1 ) {
    $byte = process_special( *CONF ) if $byte eq '\\';
    $byte = chr 0x00 if $byte =~ m/[\n ]/;
    $byte = chr 0x20 if $byte eq '_';
    $byte = ord $byte if $byte;
    $byte = ascii_to_petscii( $byte );
    push @data, sprintf "%02x", $byte;
}

my @disp = @data;
while ( @disp > 0 ) {
  my @temp = splice @disp, 0, 8;
  print map { "\$$_, " } @temp;
  print "\n";
}

open FILE, '>man,usr' or die;
binmode FILE;
foreach my $byte ( @data ) {
    my $value = chr hex $byte;
    syswrite FILE, $value, 1;
}
syswrite FILE, chr hex 0x00, 1;
syswrite FILE, chr hex 0x00, 1;
close FILE or die;

sub process_special {
    my $fh = shift;
    my $petscii_char;
    sysread CONF, $petscii_char, 4;
    $petscii_char =~ m/(\d{2})/;
    $petscii_char = chr hex $&;
    return $petscii_char;
}

sub ascii_to_petscii {
    my $ascii = shift;
    if ($ascii>=0x41 && $ascii<=0x5b) { return $ascii+0x80; }
    if ($ascii==0x5f) { return 0xa4; }
    if ($ascii==0xa4) { return 0x5f; }
    if ($ascii>=0x61 && $ascii<=0x7b) { return $ascii-0x20; }
    if ($ascii>=0xc1 && $ascii<=0xcb) { return $ascii-0x60; }
    return $ascii;
}
